// src/classes/Room.ts
import EventEmitter from "events";
var Room = class extends EventEmitter {
  id;
  type;
  clients;
  state;
  config;
  constructor(id, type, config) {
    super();
    this.id = id;
    this.type = type;
    this.clients = [];
    this.state = {};
    this.config = { ...config };
  }
  addClient(clientId) {
    this.clients.push(clientId);
  }
  removeClient(clientId) {
    this.clients = this.clients.filter((cid) => cid !== clientId);
  }
  updateState(updateFn) {
    this.state = updateFn(this.state);
    this.emit("stateUpdated", {
      roomId: this.id,
      newState: this.state
    });
  }
};

// src/classes/Server.ts
import { Server } from "socket.io";
import { v4 as uuidv4 } from "uuid";
var CybercafeServer = class {
  io;
  rooms = {};
  roomTypes = {};
  constructor(server) {
    this.io = new Server(server);
    this.io.on("connection", (socket) => {
      socket.on(
        "leaveRoom",
        (roomId) => this.leaveRoom(roomId, socket.id)
      );
      socket.on(
        "joinOrCreateRoom",
        (roomType) => this.joinOrCreateRoom(roomType, socket.id)
      );
      socket.on("joinRoomById", (roomId) => {
        this.joinRoom(roomId, socket.id);
      });
    });
  }
  joinOrCreateRoom(roomType, clientId) {
    const existingRoomId = Object.keys(this.rooms).find(
      (roomId) => this.rooms[roomId]?.type === roomType
    );
    if (existingRoomId) {
      this.joinRoom(existingRoomId, clientId);
    } else {
      const newRoomId = this.createRoom(roomType, {});
      this.joinRoom(newRoomId, clientId);
    }
  }
  define(type, constructor) {
    this.roomTypes[type] = constructor;
  }
  createRoom(roomType, config) {
    const roomId = this.generateRoomId();
    const RoomTypeConstructor = this.roomTypes[roomType];
    if (!RoomTypeConstructor) {
      throw new Error(`Room type ${roomType} is not defined.`);
    }
    const room = new RoomTypeConstructor(roomId, roomType, config);
    room.on("stateUpdated", (event) => {
      this.io.to(event.roomId).emit("stateUpdated", event.newState);
    });
    this.rooms[roomId] = room;
    return roomId;
  }
  joinRoom(roomId, clientId) {
    const room = this.rooms[roomId];
    if (room) {
      room.addClient(clientId);
      this.io.to(clientId).emit("joinedRoom", room);
    }
  }
  leaveRoom(roomId, clientId) {
    const room = this.rooms[roomId];
    room?.removeClient(clientId);
  }
  moveClientToRoom(clientId, fromRoomId, toRoomId) {
    const fromRoom = this.rooms[fromRoomId];
    const toRoom = this.rooms[toRoomId];
    if (!fromRoom) {
      throw new Error(`Room with ID ${fromRoomId} does not exist.`);
    }
    if (!toRoom) {
      throw new Error(`Room with ID ${toRoomId} does not exist.`);
    }
    fromRoom.removeClient(clientId);
    toRoom.addClient(clientId);
  }
  generateRoomId() {
    return "room_" + uuidv4();
  }
};
export {
  CybercafeServer,
  Room
};
